"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolvePathAndUpdateNode = void 0;
var path_1 = __importDefault(require("path"));
var general_utils_1 = require("./general-utils");
var ts_type_conversion_1 = require("./ts-type-conversion");
/* ****************************************************************************************************************** *
 * Node Updater
 * ****************************************************************************************************************** */
/**
 * Gets proper path and calls updaterFn to get the new node if it should be updated
 */
function resolvePathAndUpdateNode(context, node, moduleName, updaterFn) {
    var sourceFile = context.sourceFile, compilerOptions = context.compilerOptions, tsInstance = context.tsInstance, config = context.config, rootDirs = context.rootDirs, implicitExtensions = context.implicitExtensions, factory = context.factory, tsThreeInstance = context.tsThreeInstance;
    var outputPath;
    /* Have Compiler API attempt to resolve */
    var _a = tsInstance.resolveModuleName(moduleName, sourceFile.fileName, compilerOptions, tsInstance.sys), resolvedModule = _a.resolvedModule, failedLookupLocations = _a.failedLookupLocations;
    if (resolvedModule === null || resolvedModule === void 0 ? void 0 : resolvedModule.isExternalLibraryImport)
        return node;
    if (!resolvedModule) {
        var maybeURL = failedLookupLocations[0];
        if (!general_utils_1.isURL(maybeURL))
            return node;
        outputPath = maybeURL;
    }
    else {
        var extension = resolvedModule.extension, resolvedFileName = resolvedModule.resolvedFileName;
        var fileName = sourceFile.fileName;
        var filePath = tsInstance.normalizePath(path_1.default.dirname(sourceFile.fileName));
        var modulePath = path_1.default.dirname(resolvedFileName);
        /* Handle rootDirs mapping */
        if (config.useRootDirs && rootDirs) {
            var fileRootDir = "";
            var moduleRootDir = "";
            for (var _i = 0, rootDirs_1 = rootDirs; _i < rootDirs_1.length; _i++) {
                var rootDir = rootDirs_1[_i];
                if (general_utils_1.isBaseDir(rootDir, resolvedFileName) && rootDir.length > moduleRootDir.length)
                    moduleRootDir = rootDir;
                if (general_utils_1.isBaseDir(rootDir, fileName) && rootDir.length > fileRootDir.length)
                    fileRootDir = rootDir;
            }
            /* Remove base dirs to make relative to root */
            if (fileRootDir && moduleRootDir) {
                filePath = path_1.default.relative(fileRootDir, filePath);
                modulePath = path_1.default.relative(moduleRootDir, modulePath);
            }
        }
        /* Remove extension if implicit */
        outputPath = tsInstance.normalizePath(path_1.default.join(path_1.default.relative(filePath, modulePath), path_1.default.basename(resolvedFileName)));
        if (extension && implicitExtensions.includes(extension))
            outputPath = outputPath.slice(0, -extension.length);
        if (!outputPath)
            return node;
        outputPath = outputPath[0] === "." ? outputPath : "./" + outputPath;
    }
    var newStringLiteral = factory
        ? factory.createStringLiteral(outputPath)
        : ts_type_conversion_1.upSampleTsType(tsThreeInstance.createStringLiteral(outputPath));
    return updaterFn(newStringLiteral);
}
exports.resolvePathAndUpdateNode = resolvePathAndUpdateNode;
